using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using Jundroo.SimplePlanes.ModTools.Parts;
using Jundroo.SimplePlanes.ModTools.Parts.Attributes;


/// <summary>
/// A part modifier for SimplePlanes.
/// A part modifier is responsible for attaching a part modifier behaviour script to a game object within a part's hierarchy.
/// </summary>
[Serializable]
public class FlightControl : Jundroo.SimplePlanes.ModTools.Parts.PartModifier
{
    [DesignerPropertySlider(10, 70, 61, Label = "Wing Loading (lb/ft2)")]
    public double wingLoad;

    [DesignerPropertySlider(0.3f,2,18,Label = "Pitch Rate Tune (Up)")]
    public float pitchTuneP;

    [DesignerPropertySlider(0.3f,2,18, Label = "Pitch Rate Tune (Down)")]
    public float pitchTuneN;

    [DesignerPropertySlider(1, 3, 21, Label = "Over-pitch Suppressing")]
    public float pitchSuppr;

    [DesignerPropertySlider(-1.5f, 1.5f, 31, Label = "Straight Flight Trim")]
    public float trim;

    [DesignerPropertyToggleButton("1", "2", "3", "4", "5", "6", "7", Label = "Vector Thrust Group")]
    public string groupVCT;




    /// <summary>
    /// Called when this part modifiers is being initialized as the part game object is being created.
    /// </summary>
    /// <param name="partRootObject">The root game object that has been created for the part.</param>
    /// <returns>The created part modifier behaviour, or <c>null</c> if it was not created.</returns>
    public override Jundroo.SimplePlanes.ModTools.Parts.PartModifierBehaviour Initialize(UnityEngine.GameObject partRootObject)
    {
        // Attach the behaviour to the part's root object.
        var behaviour = partRootObject.AddComponent<FlightControlBehaviour>();
        return behaviour;
    }
}