/*
Breaf introduction of idea:
This program reads the player pitch input, and set the VTOL input after series of calculations. The VTOL input controls the canards rotationg of the plane.
There are mainly 3 parts: data reading, AOA calculating and controlling, which are marked at their starts and ends.

In data reading part, it reads current pitch input, aircraft velocity, altitude and cockpit rotation.

In AOA calculating part:
1.  It converts the cockpit rotation into cockpit pointing directions as vectors(including head, topside and rightside pointing).
2.  It calculates the angle between cockpit rightside and the velocity, resets the velocity data by removing its component at cockpit rightside direction, 
    in order to ignore the side sliping influence(if exists).
3.  It calculates the angle between cockpit heading and the velocity, gives it a positive or negative signal by calculate the cos angle between 
    velocity and cockpit topside pointing. This finally gives the global AOA.
4.  It calculates the angular speed (update frequence = 60hz) 

In controlling part:
1.  It calculates the max angular speed that idealy the plane can reach, this value is influenced by airspeed, altitude and wing loading, then
    judges if the current rate is higher the max rate
2a. When pitching up, if AOA is below 26, canards turn upward, while both canards and tails will suppress the pitching move if rate > maxrate.
    When AOA goes over 26(critical angle is 28), canards will turn backward to push the head down, tails stick to local AOA=28 to push the tail up.
2b. Same as 2a, just inverted
2c. If not pitching, plane will try to make the AOA close to zero.
3.  Series of data treatments, including overflow treatment, NaN(not a number) treatment, giving values to temp datas, and message display arrangement.
4.  Giving values to VTOL and Trim, and display message.
*/


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class FlightControlBehaviour : Jundroo.SimplePlanes.ModTools.Parts.PartModifierBehaviour
{
    //naming rules headings : "o-" as output, "i-" as input, "t-" as temp, "f-" as float, "d-" as double, "s-" as string, "-P" as positive, "-N" as negative
    //naming rules abbreviations : "Cpt" as cockpit, "Velo" as velocity, "Rot" as rotation, "AOA" as angle of attack
    //"Top" as the current cockpit topside pointing, same for "Right"
    float iPitch = 0, fAOA, tAOA = 0 , t1AOA = 0, t2AOA = 0, ftCpt_Velo, oVTOL, bVTOL = 0, tVTOL = 0, disVTOL, oTrim, bTrim = 0, tTrim = 0, disTrim;
    short i = 0;

    double veloX, veloY, veloZ;
    double cptX = 0, cptY = 0, cptZ = 10, topX, topY, topZ, tCptX = 0, tCptY = 0, tCptZ = 10;

    float rotX, rotY, rotZ, alt;

    double rCptX, rCptY, rCptZ, rTopX, rTopY, rTopZ;

    double cosAOA, modVelo, AOA, cosTop_Velo, cosRight_Velo, angleRight_Velo, costCpt_Velo, tCpt_Velo, costtcv;

    double cosRate, cosRS, rate, tRate = 0, disRate, maxRateP, maxRateN, rateJudgeP, rateJudgeN;

    double rightX, rightY, rightZ;

    double dVeloX, dVeloY, dVeloZ;

    string sDisplay;

    float coefCanardP, coefCanardN, coeSuppress;

    int groupVCT;

    Vector3 cptRight = Vector3.zero;
    Vector3 cptTop = Vector3.zero;
    Vector3 cockpit = Vector3.zero;
    

    void Start()
    {
        var modifier = (FlightControl)this.PartModifier;
        int.TryParse(modifier.groupVCT, out groupVCT);
    }


    void Update()
    {

        if (!ServiceProvider.Instance.GameState.IsInDesigner)
        {
            var modifier = (FlightControl)this.PartModifier;

            if (ServiceProvider.Instance.PlayerAircraft.Airspeed > 2)
            {
                //data reading starts
                iPitch = ServiceProvider.Instance.PlayerAircraft.Controls.Pitch;  //get pitch input

                veloX = ServiceProvider.Instance.PlayerAircraft.Velocity.x;         //get Velo
                veloY = ServiceProvider.Instance.PlayerAircraft.Velocity.y;
                veloZ = ServiceProvider.Instance.PlayerAircraft.Velocity.z;

                rotX = ServiceProvider.Instance.PlayerAircraft.MainCockpitRotation.x;   //get Cpt rotation
                rotY = ServiceProvider.Instance.PlayerAircraft.MainCockpitRotation.y;
                rotZ = ServiceProvider.Instance.PlayerAircraft.MainCockpitRotation.z;

                alt = ServiceProvider.Instance.PlayerAircraft.Altitude;
                if (alt > 9900)
                    alt = 9900;
                //data reading ends

                //AOA calculating starts
                //1 starts
                cockpit.Set(0f, 0f, 10f);
                cockpit = Quaternion.Euler(rotX, rotY, rotZ) * cockpit;        //Cpt heading

                cptX = cockpit.x;
                cptY = cockpit.y;
                cptZ = cockpit.z;

                cptRight.Set(10f, 0f, 0f);

                cptRight = Quaternion.Euler(rotX, rotY, rotZ) * cptRight;         //Cpt rightside pointing

                rightX = cptRight.x;
                rightY = cptRight.y;
                rightZ = cptRight.z;

                cptTop.Set(0f, 10f, 0f);

                cptTop = Quaternion.Euler(rotX, rotY, rotZ) * cptTop;       //Cpt top pointing

                topX = cptTop.x;
                topY = cptTop.y;
                topZ = cptTop.z;
                //1 ends

                //2 starts
                modVelo = Math.Sqrt(veloX * veloX + veloY * veloY + veloZ * veloZ);         //module of Velo vector

                cosRight_Velo = (veloX * rightX + veloY * rightY + veloZ * rightZ) / (modVelo * 10d);    //angle betweem Cpt rightside and Velo

                angleRight_Velo = Math.Acos(cosRight_Velo);

                dVeloX = veloX - rightX * modVelo * cosRight_Velo * 0.1d;                    //reset the Velo to ignore the side slipping effect
                dVeloY = veloY - rightY * modVelo * cosRight_Velo * 0.1d;
                dVeloZ = veloZ - rightZ * modVelo * cosRight_Velo * 0.1d;

                modVelo = modVelo * Math.Sin(angleRight_Velo);

                //2 ends

                //3 starts
                cosAOA = (dVeloX * cptX + dVeloY * cptY + dVeloZ * cptZ) / (modVelo * 10d);         // cos of angle between Cpt heading and Velo

                cosTop_Velo = (dVeloX * topX + dVeloY * topY + dVeloZ * topZ) / (modVelo * 10d);    // cos of angle between Cpt top and Velo(for signal)

                AOA = Math.Acos(cosAOA);                              //calculate AOA
                AOA = AOA * 180d / Math.PI;
                //3 ends

                //4 starts
                if (cosTop_Velo > 0d)                                       //AOA signal 
                    AOA = -AOA;

                if (Double.IsNaN(AOA))
                    AOA = 0;

                fAOA = (float)AOA;

                cosRate = (cptX * tCptX + cptY * tCptY + cptZ * tCptZ) / 100;  //angular speed calculating, same method as AOA
                rate = Math.Acos(cosRate);
                cosRS = (topX * tCptX + topY * tCptY + topZ * tCptZ) / 100;
                rate = rate * 10800 / Math.PI;
                if (cosRS > 0)
                    rate = -rate;

                if (Math.Abs(Math.Abs(tRate / rate) - 2) < 0.2)   //sometimes the rate calculated will suddenly go into half, here for covering that
                    rate = rate * 2;

                if (Double.IsNaN(rate))
                    rate = tRate;
                //4 ends
                //AOA calculating ends
                
                //controlling starts
                //1 starts

                maxRateP = ServiceProvider.Instance.PlayerAircraft.Airspeed * Math.Sqrt(1 - alt / 10000) * 1.18 * modifier.pitchTuneP * 180 / Math.PI / modifier.wingLoad / 4.79;
                maxRateN = ServiceProvider.Instance.PlayerAircraft.Airspeed * Math.Sqrt(1 - alt / 10000) * -0.8 * modifier.pitchTuneN * 180 / Math.PI / modifier.wingLoad / 4.79;
                //a magic formula that gives the max angular speed for an ideal constant pitching, the altitude influence is by guessing but it works quite well

                rateJudgeP = 1 - rate / maxRateP;
                rateJudgeN = 1 - rate / maxRateN;

                coefCanardP = 1;   //initiallizing 
                coefCanardN = 1;
                //1 ends

                //2 starts
                //2a starts
                if (iPitch < 0f)                               //when pitching up
                {
                    if (!ServiceProvider.Instance.PlayerAircraft.Controls.GetActivationGroupState(groupVCT))  //activate vector thrust
                        ServiceProvider.Instance.PlayerAircraft.Controls.ToggleActivationGroup(groupVCT);
                    if (rateJudgeP < 0 )  //when actual rate > max rate
                    {
                        oTrim = (-fAOA + 28f) / 45f;   //tails suppress
                        coefCanardP = 0;               //canards neutral
                    }
                    else               //when actual rate < max rate
                    {
                        oTrim = (-modifier.trim + iPitch * (28f - fAOA ) * (float)rateJudgeP) / 45f;  //tails push down
                        coefCanardP = (float)rateJudgeP;                                              //set coefficient for canards
                    }

                    if (fAOA < 25f)  //when AOA < 25 (28 is the ideal angle)
                    {
                        oVTOL = (modifier.trim + (-iPitch) * ((-modifier.trim -28f) * iPitch - fAOA) * coefCanardP) / 45f; //canards push up
                    }
                    else      //when AOA > 25
                    {
                        coeSuppress = fAOA / tAOA;          //coeffitient > 1 if AOA increasing, < 1 if decreasing  
                        oVTOL = (-iPitch) * (25f - fAOA) / 45f * modifier.pitchSuppr;   //canards suppress
                        oVTOL = oVTOL * (float)Math.Pow(coeSuppress, 5);         //canards suppress more strongly if AOA increasing, less if decreasing
                        
                        oTrim = (-iPitch) * (28f - fAOA) / 45f;             //tails go to AOA to push down, suppress
                    }
                }
                //2a ends
                //2b starts
                else if (iPitch > 0f)                            //when pitching down, everything same like pitchiing up, but no vector thrust
                {
                    if (rateJudgeN < 0)
                    {
                        oTrim = (-fAOA - 17f) / 45f;
                        coefCanardN = 0;
                    }
                    else
                    {
                        oTrim = -modifier.trim / 45f;
                        coefCanardP = (float)rateJudgeN;
                    }
                    if (fAOA > -15f)
                    {
                        oVTOL = (-iPitch) * (fAOA + (17f * iPitch)) * coefCanardP / 45f;
                        if (ServiceProvider.Instance.PlayerAircraft.Controls.GetActivationGroupState(groupVCT))  //insure that vector thrust is off
                            ServiceProvider.Instance.PlayerAircraft.Controls.ToggleActivationGroup(groupVCT);
                    }
                    else
                    {
                        coeSuppress = fAOA / tAOA;
                        oVTOL = (-iPitch) * (fAOA + 15f) / 45f * modifier.pitchSuppr;
                        oVTOL = oVTOL * (float)Math.Pow(coeSuppress, 5);
                        oTrim = (-iPitch) * (17f + fAOA) / 45f;
                        if (!ServiceProvider.Instance.PlayerAircraft.Controls.GetActivationGroupState(groupVCT))  //if -AOA too high, vector thrust on 
                            ServiceProvider.Instance.PlayerAircraft.Controls.ToggleActivationGroup(groupVCT);
                    }
                }
                //2b ends
                //2c starts
                else         //if not pitching
                {                                               
                    oTrim = - modifier.trim / 45f;           //tails stick to trim value
                    oVTOL = (-fAOA + modifier.trim) / 45f;   //canards try to make plane going straight with trim value

                    if (fAOA > 28f && fAOA < 90f)            //if AOA too high, tails suppress
                    {
                        oTrim = (28f - fAOA) / 45f;
                    }
                    else if (fAOA < -17.5f && fAOA > -90)   //if -AOA too high, tails suppress 
                    {
                        oTrim = (-17.5f + fAOA) / 45f;
                    }

                    if ((fAOA > 60f && fAOA < 90f) || (fAOA < -45f && fAOA > -90f))  //if AOA too too high, vector thrust on
                    {
                        if (!ServiceProvider.Instance.PlayerAircraft.Controls.GetActivationGroupState(groupVCT))
                            ServiceProvider.Instance.PlayerAircraft.Controls.ToggleActivationGroup(groupVCT);
                    }
                    else if (ServiceProvider.Instance.PlayerAircraft.Controls.GetActivationGroupState(groupVCT))  //insure that vector thrust is off in other cases
                        ServiceProvider.Instance.PlayerAircraft.Controls.ToggleActivationGroup(groupVCT);

                    if (fAOA > modifier.trim && fAOA < 60f && rate < 0)  //same as above for -AOA
                    {
                        oTrim = -modifier.trim / 45f;
                        oVTOL = modifier.trim / 45f;
                        if (ServiceProvider.Instance.PlayerAircraft.Controls.GetActivationGroupState(groupVCT))
                            ServiceProvider.Instance.PlayerAircraft.Controls.ToggleActivationGroup(groupVCT);
                    }
                    else if (fAOA < modifier.trim && fAOA > -60f && rate > 0) //same as above for -AOA
                    {
                        oTrim = -modifier.trim / 45f;
                        oVTOL = modifier.trim / 45f;
                        if (ServiceProvider.Instance.PlayerAircraft.Controls.GetActivationGroupState(groupVCT))
                            ServiceProvider.Instance.PlayerAircraft.Controls.ToggleActivationGroup(groupVCT);
                    }
                }
                //2c ends
                //2 ends

                //3 starts
                if (oVTOL > 1.0f)    //in case that values overflow
                    oVTOL = 1.0f;

                if (oVTOL < -1.0f)
                    oVTOL = -1.0f;

                if (oTrim > 1.0f)
                    oTrim = 1.0f;

                if (oTrim < -1.0f)
                    oTrim = -1.0f;

                tAOA = t1AOA;     //set values to temps
                t1AOA = fAOA;

                tCptX = cptX;
                tCptY = cptY;
                tCptZ = cptZ;

                tRate = rate;

                oVTOL = oVTOL * 0.7f + tVTOL * 0.3f;   //little formula to smoothen the moves
                oTrim = oTrim * 0.7f + tTrim * 0.3f;

                if (Single.IsNaN(oVTOL))   //in case that values are NaN
                    oVTOL = bVTOL;

                if (Single.IsNaN(oTrim))
                    oTrim = bTrim;

                tVTOL = oVTOL;
                tTrim = oTrim;

                bVTOL = oVTOL;        //backup datas for NaN
                bTrim = oTrim;

                disVTOL = oVTOL * 45;  //values displayed for canards and tails
                disTrim = oTrim * 45;

                if (!ServiceProvider.Instance.PlayerAircraft.Controls.GetActivationGroupState(8))  //when flight control is turned off, pitch input directly controls
                {
                    oVTOL = -iPitch;
                    oTrim = iPitch;
                }

                sDisplay = "AOA: " + fAOA.ToString("N1") + "  Canard: " + disVTOL.ToString("N1") //display messages arrangement
                + "  Tail: " + disTrim.ToString("N1") + "  PitchRate: " + rate.ToString("N1");
                //3 ends
                //4 starts
                ServiceProvider.Instance.PlayerAircraft.Controls.Vtol = oVTOL;     //finally give values to canards and tails
                ServiceProvider.Instance.PlayerAircraft.Controls.Trim = oTrim;

                i++;
                if (i == 6)
                {
                    ServiceProvider.Instance.GameWorld.ShowStatusMessage(sDisplay, 0.1f); //display message every 6 frames
                    i = 0;
                }
                //4 ends
                //controlling ends

            }
        }
    }
}